
#ifdef WIN32
#pragma comment(lib,"Foundation.lib")
#pragma comment(lib,"OpenMaya.lib")
#pragma comment(lib,"OpenMayaAnim.lib")
#define NT_APP
#endif

#include <maya/MAnimControl.h>
#include <maya/MFileIO.h>
#include <maya/MFloatMatrix.h>
#include <maya/MFloatVector.h>
#include <maya/MFloatVectorArray.h>
#include <maya/MLibrary.h>
#include <maya/MMatrix.h>
#include <maya/MPlug.h>
#include <maya/MPoint.h>
#include <maya/MPointArray.h>

#include <maya/MFnMesh.h>
#include <maya/MFnTransform.h>

#include <maya/MItDependencyNodes.h>
#include <maya/MItMeshPolygon.h>
#include <maya/MItMeshVertex.h>

#include <GL/glut.h>

#include "FrameTimer.h"
#include "FileDialog.h"

//##########################################################################	Animation Stuff
//--------------------------------------------------------------------------

/// the start time (in seconds) of the animation
float g_AnimStartTime=1;

/// the end time (in seconds) of the animation
float g_AnimEndTime=24;

/// the current time (in seconds) that we are into the animation
float g_AnimCurrentTime=1;

//--------------------------------------------------------------------------
/// \brief	This function queries the time range over which the animation 
///			will run.
///
void InitAnimTime()
{
	g_AnimStartTime = MAnimControl::minTime().as(MTime::kSeconds);
	g_AnimEndTime = MAnimControl::maxTime().as(MTime::kSeconds);
	g_AnimCurrentTime = g_AnimStartTime;
}

//--------------------------------------------------------------------------
/// \brief	This function updates the current animation time 
/// \param	
/// \return	
///
void UpdateAnimTime(float dt)
{
	g_AnimCurrentTime += dt;
	while (g_AnimCurrentTime > g_AnimEndTime) {
		g_AnimCurrentTime -= ( g_AnimEndTime - g_AnimStartTime );
	}

	// set the current time
	MAnimControl::setCurrentTime( MTime(g_AnimCurrentTime,MTime::kSeconds) );
}









//##########################################################################	Rendering Stuff
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
/// \brief	This function recieves a handle to a mesh and then draws the 
///			polygons using openGL
/// \param	obj	-	a handle to the mesh
///
void DrawMesh(const MObject& obj) 
{
	// attach a function set to the mesh handle so that we can 
	// access the mesh data.
	MFnMesh fn(obj);

	MPointArray vts;
	MFloatVectorArray nmls;
	MFloatArray u_coords;
	MFloatArray v_coords;

	// get the vertex data
	fn.getPoints(vts);

	// get the surface normals
	fn.getNormals(nmls);

	// get the texturing coords
	fn.getUVs(u_coords,v_coords);



	// attach a polygon iterator to the mesh so that we can walk
	// through the mesh faces.
	//
	MItMeshPolygon itPoly(obj);

	// write each tex coord
	while(!itPoly.isDone()) 
	{
		// how many vertices do we have on the face?
		int vc = itPoly.polygonVertexCount();

		// start drawing a polygon
		glBegin(GL_POLYGON);

		// print all vertex,normal and uv indices
		for(int i=0;i<vc;++i) {

			int v_index,n_index;

			// get the indicies for this face point into the vertex & normal arrays
			v_index = itPoly.vertexIndex(i);
			n_index = itPoly.normalIndex(i);

			// get the correct vertex for this face point
			MPoint V = vts[ v_index ];

			// get the normal for this point on the face
			MFloatVector N = nmls[ n_index ];

			// only specify the texturing coords if they are used
			if(u_coords.length()>0)
			{
				int uv_index;
				itPoly.getUVIndex(i,uv_index);

				// get the u and v texture coord
				float u = u_coords[ uv_index ];
				float v = v_coords[ uv_index ];

		        glTexCoord2f(u,v);
			}

			glNormal3f(N.x,N.y,N.z);
			glVertex3f(V.x,V.y,V.z);
		}

		glEnd();

		itPoly.next();
	}
}

//--------------------------------------------------------------------------
/// \brief	Takes the matrix from Maya and applies it to the openGL modelview
///			matrix stack.
/// \param	M	-	the matrix transform
///
void ApplyMatrix(MMatrix& M)
{
	float mat[16];

	mat[0]=M(0,0);
	mat[1]=M(0,1);
	mat[2]=M(0,2);
	mat[3]=M(0,3);

	mat[4]=M(1,0);
	mat[5]=M(1,1);
	mat[6]=M(1,2);
	mat[7]=M(1,3);

	mat[8]=M(2,0);
	mat[9]=M(2,1);
	mat[10]=M(2,2);
	mat[11]=M(2,3);

	mat[12]=M(3,0);
	mat[13]=M(3,1);
	mat[14]=M(3,2);
	mat[15]=M(3,3);

	glMultMatrixf(mat);
}

//--------------------------------------------------------------------------
/// \brief	This function recursively descends up the hierarchy to make a
///			parent transform.
/// \param	obj	-	the parent
/// \return	the world space matrix
///
MMatrix BuildTransform(const MObject& obj)
{
	MFnTransform fn(obj);

	// get matrix transform for this bone
	MMatrix M = fn.transformationMatrix();

	if (fn.parentCount()>0) {
		return M* BuildTransform(fn.parent(0)) ;
	}
	return M;	
}

//--------------------------------------------------------------------------
/// \brief	This function simply traverses the scene and then renders all of 
///			the meshes it finds along the way.
///
void DrawAllMeshes()
{
	MItDependencyNodes it(MFn::kMesh);

	for( ; !it.isDone(); it.next() )
	{
		// attach a function set to this mesh so that we can find
		// out if it is a history item.
		//
		MFnDagNode fn(it.item());

		// only draw non-history
		if (!fn.isIntermediateObject() )
		{
			// loop through each mesh instance and draw it
			for(int i=0;i<fn.parentCount();++i)
			{

				glPushMatrix();

					// get the transform matrix to put the mesh into world space
					MMatrix XForm = BuildTransform( fn.parent(i) );

					// apply the matrix transform to put this shape into world space.
					ApplyMatrix(XForm);

					// now draw the mesh using local space coords.
					DrawMesh(it.item());

				glPopMatrix();
			}
		}
	}
}










//##########################################################################	Camera Movement Stuff
//--------------------------------------------------------------------------

float g_fZoom = 15.0f;
float g_fRotx = 0;
float g_fRoty = 0.001f;
float g_fTrx  = 0;
float g_fTry  = 0;
int g_iLastx=0;
int g_iLasty=0;
unsigned char g_Buttons[3] = {0};

//--------------------------------------------------------------------------
/// \brief	glut mouse motion handler
///
void Motion(int x,int y)
{
	int diffx=x-g_iLastx;
	int diffy=y-g_iLasty;
	g_iLastx=x;
	g_iLasty=y;

	if( g_Buttons[0] && g_Buttons[1] )
	{
		g_fZoom -= (float) 0.05f * diffx;
	}
	else
	if( g_Buttons[0] )
	{
		g_fRotx += (float) 0.5f * diffy;
		g_fRoty += (float) 0.5f * diffx;		
	}
	else
	if( g_Buttons[1] )
	{
		g_fTrx += (float) 0.05f * diffx;
		g_fTry -= (float) 0.05f * diffy;
	}
}

//--------------------------------------------------------------------------
/// \brief	glut mouse button handler
///
void Mouse(int b,int s,int x,int y)
{
	g_iLastx=x;
	g_iLasty=y;
	switch(b)
	{
	case GLUT_LEFT_BUTTON:
		g_Buttons[0] = ((GLUT_DOWN==s)?1:0);
		break;

	case GLUT_MIDDLE_BUTTON:
		g_Buttons[1] = ((GLUT_DOWN==s)?1:0);
		break;

	case GLUT_RIGHT_BUTTON:
		g_Buttons[2] = ((GLUT_DOWN==s)?1:0);
		break;

	default:
		break;		
	}
}





//##########################################################################	Glut Stuff
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
/// \brief	Initialises any openGL stuff
///
void Init() 
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
	InitFrameTimer();
	InitAnimTime();
}


//--------------------------------------------------------------------------
/// \brief	Draws the scene
///
void Display()
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(0,0,-g_fZoom);
	glTranslatef(g_fTrx,g_fTry,0);
	glRotatef(g_fRotx,1,0,0);
	glRotatef(g_fRoty,0,1,0);	

	glPushMatrix();

	DrawAllMeshes();

	glPopMatrix();

	glutSwapBuffers();
}

//--------------------------------------------------------------------------
/// \brief	Called when the screen resolution changes
/// \param	w	-	the new screen width
/// \param	h	-	the new screen height
/// 
void Reshape(int x, int y)
{
	if(y==0) y = 1;
	glViewport(0,0,x,y);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45,(float)x/y,0.1,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}

//--------------------------------------------------------------------------
/// \brief	idle processing
///
void Idle()
{
	SortFrameTimer();
	UpdateAnimTime(FrameTime());
	glutPostRedisplay();
}

//--------------------------------------------------------------------------
/// \brief	Exit callback
///
void OnExit()
{
	MLibrary::cleanup();
}





//##########################################################################	Main
//--------------------------------------------------------------------------


#ifdef WIN32
int main(int argc,char** argv)
{

#else
int g_argc=0;
char** g_argv=0;
int main(int argc,char** argv)
{
	g_argc = argc;
	g_argv = argv;

#endif
	std::string infile;
	if( argc == 2 )
		infile = argv[1];
	else
	{
		char buffer[512] = {0};
		if( !OpenFileDialog(buffer) ) {
			return 0;
		}
		infile = buffer;
	}

	MLibrary::initialize(argv[0]);

	if (MFileIO::open(infile.c_str())!= MS::kSuccess) {
		std::cerr << "[ERROR] Unable to open file " << infile.c_str() << std::endl;
		MLibrary::cleanup();
		return 0;
	}

	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGBA|GLUT_DEPTH);
	glutInitWindowSize(640,480);
	glutInitWindowPosition(100,100);
	glutCreateWindow("Maya Binary Viewer");


	glutDisplayFunc(Display);
	glutReshapeFunc(Reshape);
	glutIdleFunc(Idle);
	glutMouseFunc(Mouse);		/* set the mouse press handler function */
	glutMotionFunc(Motion);		/* set the mouse motion handler */

	// register an exit function to clean the maya API
	atexit(OnExit);

	Init();

	glutMainLoop();
}

